package net.sf.snmpadaptor4j.example;

import java.lang.management.ManagementFactory;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import net.sf.snmpadaptor4j.SnmpAdaptor;
import net.sf.snmpadaptor4j.example.mbean.Example;

/**
 * JMX manager.
 * @author Jean-Philippe MINETTI
 */
public final class Jmx {

	/**
	 * MBean server.
	 */
	private final MBeanServer mBeanServer = ManagementFactory.getPlatformMBeanServer();

	/**
	 * SNMP adaptor.
	 */
	private final SnmpAdaptor snmpAdaptor = new SnmpAdaptor(getClass().getResource("/snmp.xml"), true);

	/**
	 * Name for the SNMP adaptor.
	 */
	private ObjectName snmpAdaptorMBeanName = null;

	/**
	 * Name for the system informations MBean.
	 */
	private ObjectName systemInfoMBeanName = null;

	/**
	 * Name for the read only MBean.
	 */
	private ObjectName exampleReadOnlyMBeanName = null;

	/**
	 * Name for the writable MBean.
	 */
	private ObjectName exampleWritableMBeanName = null;

	/**
	 * Constructor.
	 * @throws Exception Exception if an error occurred.
	 */
	public Jmx () throws Exception {
		super();
	}

	/**
	 * Starts JMX.
	 * @throws Exception Exception if an error occurred.
	 */
	public void start () throws Exception {
		this.exampleReadOnlyMBeanName = new ObjectName("net.sf.snmpadaptor4j.example:type=Example,name=readOnly");
		this.mBeanServer.registerMBean(new Example(), this.exampleReadOnlyMBeanName);
		this.snmpAdaptorMBeanName = new ObjectName("net.sf.snmpadaptor4j.example:adaptor=SnmpAdaptor");
		this.mBeanServer.registerMBean(this.snmpAdaptor, this.snmpAdaptorMBeanName);
		this.snmpAdaptor.start();
		this.systemInfoMBeanName = new ObjectName("net.sf.snmpadaptor4j.example:type=SystemInfo");
		this.mBeanServer.registerMBean(this.snmpAdaptor.getSystemInfo(), this.systemInfoMBeanName);
		this.exampleWritableMBeanName = new ObjectName("net.sf.snmpadaptor4j.example:type=Example,name=writable");
		this.mBeanServer.registerMBean(new Example(), this.exampleWritableMBeanName);
	}

	/**
	 * Stops JMX.
	 * @throws Exception Exception if an error occurred.
	 */
	public void stop () throws Exception {
		this.mBeanServer.unregisterMBean(this.exampleWritableMBeanName);
		this.mBeanServer.unregisterMBean(this.systemInfoMBeanName);
		this.mBeanServer.unregisterMBean(this.snmpAdaptorMBeanName);
		this.mBeanServer.unregisterMBean(this.exampleReadOnlyMBeanName);
		this.snmpAdaptor.stop();
	}

}